body {
    margin: 0;

    font-family: sans-serif;
    background-color: #222;
}

header {
    padding: 0.75em;

    text-align: center;
    font-family: monospace;
    font-size: 3.5em;
    color: #eee;
}

main {
    margin: auto;
    padding: 1.5em;
    max-width: 70em;
    min-height: 20em;

    color: #444;
    background-color: #fff;
}

main a {
    color: #362;
    transition: color 0.1s ease-in-out;
}

main a:hover {
    color: #241;
}

/* Headline */
main h1 {
    margin: 0;
    padding: 0 0.1em 0.1em 0.1em;

    font-size: 2em;
    border-bottom: 0.25em solid #444; /* Underline */
}

/* Breadcrumb */
main nav {
    margin-bottom: 1.5em;
    padding: 0.3em;

    font-style: italic;
    color: #555;
    border: 0.1em solid #444;
    border-top: none;
}

footer {
    padding: 1em;

    text-align: center;
    font-family: monospace;
    font-weight: bold;
    color: #fff;
}

footer em {
    font-weight: normal;
    font-style: normal;
}

footer a {
    color: #fff;
    transition: color 0.1s ease-in-out;
}

footer a:hover {
    color: #bbb;
}
